package com.ruoyi.system.support;


import com.ruoyi.system.constant.CoalConstant;
import com.ruoyi.system.mapper.SalaryMapper;
import com.ruoyi.system.model.newrule.NewRuleRequestModel;
import com.ruoyi.system.model.salary.ConfigModel;
import com.ruoyi.system.model.salary.SalaryExcelModel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ReflectionUtils;


import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component
public class CurrentRuleSalarySupport {

    public final static Map<String, Function<List<NewRuleRequestModel>, List<SalaryExcelModel>>> MINE_MAP = new HashMap<>();

    static {
        MINE_MAP.put(CoalConstant.TS_MINE, CurrentRuleSalarySupport::getTsMineSalaryData);
        MINE_MAP.put(CoalConstant.LJT_MINE, CurrentRuleSalarySupport::getLjtMineSalaryData);
        MINE_MAP.put(CoalConstant.LX_MINE, CurrentRuleSalarySupport::getLxMineSalaryData);
        MINE_MAP.put(CoalConstant.DHT_MINE, CurrentRuleSalarySupport::getDhtMineSalaryData);
        MINE_MAP.put(CoalConstant.SH_MINE, CurrentRuleSalarySupport::getShMineSalaryData);
        MINE_MAP.put(CoalConstant.FK_MINE, CurrentRuleSalarySupport::getFKMineSalaryData);
        MINE_MAP.put(CoalConstant.HF_MINE, CurrentRuleSalarySupport::getHfMineSalaryData);
        MINE_MAP.put(CoalConstant.QJY_MINE, CurrentRuleSalarySupport::getQjyMineSalaryData);
        MINE_MAP.put(CoalConstant.YF_MINE, CurrentRuleSalarySupport::getYfMineSalaryData);
    }


    private static SalaryMapper salaryMapper;

    @Autowired
    public void setSalaryMapper(SalaryMapper salaryMapper) {
        CurrentRuleSalarySupport.salaryMapper = salaryMapper;
    }


    /**
     * 云飞
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getYfMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 钱家营矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getQjyMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 宏丰
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getHfMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 范各庄矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getFKMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 单侯
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getShMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }


    /**
     * 唐山矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getTsMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);

    }

    /**
     * 吕家坨矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */

    private static List<SalaryExcelModel> getLjtMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }

    /**
     * 林西矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getLxMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        newRuleSalaryRequestModelList = newRuleSalaryRequestModelList.stream().map(v -> {
            v.setColoumn(v.getColoumn().replaceAll(StringUtils.SPACE, StringUtils.EMPTY));
            v.setRow(v.getRow().replaceAll(StringUtils.SPACE, StringUtils.EMPTY));
            v.setMineid(v.getMineid().replaceAll(StringUtils.SPACE, StringUtils.EMPTY));
            v.setValue(v.getValue().replaceAll(StringUtils.SPACE, StringUtils.EMPTY));
            v.setWageMonth(v.getWageMonth().replaceAll(StringUtils.SPACE, StringUtils.EMPTY));
            return v;
        }).collect(Collectors.toList());
        List<SalaryExcelModel> salaryExcelModelList = getSalaryMapping(newRuleSalaryRequestModelList);
        salaryExcelModelList.forEach(v -> {
            if (StringUtils.isNotBlank(v.getSumSalary())) {
                String sumSalary = v.getSumSalary();
                v.setEdufFees(getBigDecimalData(sumSalary, "0.015"));
                v.setLaborUnionDues(getBigDecimalData(sumSalary, "0.02"));
            }
        });
        return salaryExcelModelList.stream().filter(v -> !StringUtils.containsAny(v.getOrg(), "小计", "总计", "合计")).collect(Collectors.toList());
    }

    /**
     * 东欢坨矿
     *
     * @param newRuleSalaryRequestModelList
     * @return
     */
    private static List<SalaryExcelModel> getDhtMineSalaryData(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        return getSalaryMapping(newRuleSalaryRequestModelList);
    }


    public static List<SalaryExcelModel> getSalaryMapping(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        String orgCode = newRuleSalaryRequestModelList.stream().findFirst().get().getMineid();
        List<SalaryExcelModel> salaryExcelModelList = getSalaryExcelModelList(newRuleSalaryRequestModelList);
        List<SalaryExcelModel> finalList = new ArrayList<>();
        Map<String, List<SalaryExcelModel>> collect;
        //按人员的
        if (StringUtils.equalsAny(orgCode, "010105", "011204")) {
            collect = salaryExcelModelList.stream().collect(Collectors.groupingBy(SalaryExcelModel::getPersonNo));
        } else {
            //按部门的
            collect = salaryExcelModelList.stream().collect(Collectors.groupingBy(SalaryExcelModel::getOrg));
        }
        collect.forEach((k, v) -> {
            SalaryExcelModel salaryExcelModel = mergeData(v);
            finalList.add(salaryExcelModel);
        });
        return finalList;
    }


    public static SalaryExcelModel mergeData(List<SalaryExcelModel> dataList) {
        return dataList.stream().collect(Collectors.collectingAndThen(Collectors.toMap(data -> data.getBelongToCoal() + data.getOrg() + data.getSalaryDate(), data -> data, (existingData, newData) -> {
            existingData.setPersonCount(StringUtils.isEmpty(existingData.getPersonCount()) ? newData.getPersonCount() : existingData.getPersonCount());
            existingData.setPersonNo(StringUtils.isEmpty(existingData.getPersonNo()) ? newData.getPersonNo() : existingData.getPersonNo());
            existingData.setPersonName(StringUtils.isEmpty(existingData.getPersonName()) ? newData.getPersonName() : existingData.getPersonName());
            existingData.setMedicalInsurance(StringUtils.isEmpty(existingData.getMedicalInsurance()) ? newData.getMedicalInsurance() : existingData.getMedicalInsurance());
            existingData.setOldInsurance(StringUtils.isEmpty(existingData.getOldInsurance()) ? newData.getOldInsurance() : existingData.getOldInsurance());
            existingData.setUnemploymentInsurance(StringUtils.isEmpty(existingData.getUnemploymentInsurance()) ? newData.getUnemploymentInsurance() : existingData.getUnemploymentInsurance());
            existingData.setSalary(StringUtils.isEmpty(existingData.getSalary()) ? newData.getSalary() : existingData.getSalary());
            existingData.setMedicalSubsidy(StringUtils.isEmpty(existingData.getMedicalSubsidy()) ? newData.getMedicalSubsidy() : existingData.getMedicalSubsidy());
            existingData.setSumSalary(StringUtils.isEmpty(existingData.getSumSalary()) ? newData.getSumSalary() : existingData.getSumSalary());
            existingData.setAnnuity(StringUtils.isEmpty(existingData.getAnnuity()) ? newData.getAnnuity() : existingData.getAnnuity());
            existingData.setSingleSupplyMedicalInsurance(StringUtils.isEmpty(existingData.getSingleSupplyMedicalInsurance()) ? newData.getSingleSupplyMedicalInsurance() : existingData.getSingleSupplyMedicalInsurance());
            existingData.setSupplyMedicalInsurance(StringUtils.isEmpty(existingData.getSupplyMedicalInsurance()) ? newData.getSupplyMedicalInsurance() : existingData.getSupplyMedicalInsurance());
            existingData.setBirthInsurance(StringUtils.isEmpty(existingData.getBirthInsurance()) ? newData.getBirthInsurance() : existingData.getBirthInsurance());
            existingData.setInjuryInsurance(StringUtils.isEmpty(existingData.getInjuryInsurance()) ? newData.getInjuryInsurance() : existingData.getInjuryInsurance());
            existingData.setCompanyHouseFund(StringUtils.isEmpty(existingData.getCompanyHouseFund()) ? newData.getCompanyHouseFund() : existingData.getCompanyHouseFund());
            existingData.setHouseFund(StringUtils.isEmpty(existingData.getHouseFund()) ? newData.getHouseFund() : existingData.getHouseFund());
            existingData.setLaborUnionDues(StringUtils.isEmpty(existingData.getLaborUnionDues()) ? newData.getLaborUnionDues() : existingData.getLaborUnionDues());
            existingData.setEdufFees(StringUtils.isEmpty(existingData.getEdufFees()) ? newData.getEdufFees() : existingData.getEdufFees());
            existingData.setOneChild(StringUtils.isEmpty(existingData.getOneChild()) ? newData.getOneChild() : existingData.getOneChild());
            existingData.setLunch(StringUtils.isEmpty(existingData.getLunch()) ? newData.getLunch() : existingData.getLunch());
            existingData.setLongAccountInsurance(StringUtils.isEmpty(existingData.getLongAccountInsurance()) ? newData.getLongAccountInsurance() : existingData.getLongAccountInsurance());
            existingData.setHonorMoney(StringUtils.isEmpty(existingData.getHonorMoney()) ? newData.getHonorMoney() : existingData.getHonorMoney());
            existingData.setWarmSubsidy(StringUtils.isEmpty(existingData.getWarmSubsidy()) ? newData.getWarmSubsidy() : existingData.getWarmSubsidy());
            existingData.setHeatPreventSubsidy(StringUtils.isEmpty(existingData.getHeatPreventSubsidy()) ? newData.getHeatPreventSubsidy() : existingData.getHeatPreventSubsidy());
            return existingData;
        }), map -> map.values().iterator().next()));
    }

    private static List<SalaryExcelModel> getSalaryExcelModelList(List<NewRuleRequestModel> newRuleSalaryRequestModelList) {
        String mineId = newRuleSalaryRequestModelList.stream().findFirst().get().getMineid();
        List<ConfigModel> configModelList = salaryMapper.getConfig();
        List<SalaryExcelModel> salaryExcelModelList = new ArrayList<>();
        Map<String, List<NewRuleRequestModel>> collect = newRuleSalaryRequestModelList.stream().collect(Collectors.groupingBy(NewRuleRequestModel::getRow));
        collect.forEach((key, value) -> {
            SalaryExcelModel salaryExcelModel = new SalaryExcelModel();
            for (NewRuleRequestModel newRuleRequestModel : value) {
                for (ConfigModel configModel : configModelList) {
                    if (StringUtils.contains(configModel.getColName(), newRuleRequestModel.getColoumn())) {
                        Field field;
                        try {
                            field = SalaryExcelModel.class.getDeclaredField(configModel.getKeyName());
                            ReflectionUtils.makeAccessible(field);
                            ReflectionUtils.setField(field, salaryExcelModel, newRuleRequestModel.getValue());
                        } catch (NoSuchFieldException e) {
                            throw new RuntimeException(e);
                        }
                    }
                }
                salaryExcelModel.setBelongToCoal(newRuleRequestModel.getMineid());
                salaryExcelModel.setSalaryDate(newRuleRequestModel.getWageMonth());
            }
            salaryExcelModelList.add(salaryExcelModel);
        });
        if (StringUtils.equals("010105", mineId)) {
            salaryExcelModelList.forEach(v -> v.setSourceExpenses(getQJYSource(v)));
        }
        return salaryExcelModelList;
    }

    private static String getQJYSource(SalaryExcelModel salaryExcelModel) {
        String deptAttr = salaryExcelModel.getDeptAttr();
        String org = salaryExcelModel.getOrg();
        if (StringUtils.equals("机关", deptAttr) && !StringUtils.equals("洗煤厂", org)) {
            return "管理费用-管理费";
        }
        if (!StringUtils.equals("机关", deptAttr)) {
            if (StringUtils.equals("洗煤厂", org)) {
                return "基本生产-洗煤";
            } else if (StringUtils.containsAny(org, "后勤", "宾馆")) {
                return "制造费用-原煤";
            } else if (StringUtils.containsAny(org, "人资")) {
                return "管理费用-劳动保险费";
            } else if (StringUtils.contains(org, "研发")) {
                return "研发入相应工资";
            } else if (StringUtils.contains(org, "兴隆洗煤厂")) {
                return "管理费用-管理费（部门-兴隆洗煤厂）";
            } else {
                return "基本生产-原煤";
            }
        }
        return StringUtils.EMPTY;
    }

    public static String getBigDecimalData(String sumSalary, String coefficient) {
        BigDecimal multiply = new BigDecimal(sumSalary).multiply(new BigDecimal(coefficient));
        return multiply.setScale(2, RoundingMode.HALF_UP).toString();
    }
}
